==> neper-1.8.0.tgz: neper-1.8.0/src/neper_t/Miscellaneous/Miscellaneous2.c <==
/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"Miscellaneous.h"

void
DNeighSize (struct DNEIGH *pDNeigh)
{
  int i, j, k;
  int ver, poly;
  double *tmp;
  int **DFaceVer;
  double *s = (*pDNeigh).Size;

  struct GERMSET GermSet;
  struct TESS Tess;

  if ((*pDNeigh).N <= 0)
    {
      MessageWNC (2, 72, "The domain has no germ.");
      abort ();
    }

  if (((*pDNeigh).Size[1] >= (*pDNeigh).Size[2])
      || ((*pDNeigh).Size[3] >= (*pDNeigh).Size[4])
      || ((*pDNeigh).Size[5] >= (*pDNeigh).Size[6]))
    {
      MessageWNC (2, 72, "Bad domain size.");
      abort ();
    }

  /* Building GermSet */
  GermSet.N = (*pDNeigh).N;
  /* GermSet.Id=(*pDNeigh).Id; */
  GermSet.GermsCoo = ut_alloc_2d (GermSet.N + 1, 4);
  for (i = 1; i <= GermSet.N; i++)
    {
      for (j = 1; j <= 3; j++)
	GermSet.GermsCoo[i][j] = (*pDNeigh).GermsCoo[i][j];
    }

  /* Building tessellation to calculate the neighbourhood zone. */
  CreateTess (GermSet, (*pDNeigh).Size, &Tess);

  DFaceVer = ut_alloc_2d_int (7, Tess.VerQty + 1);
  SearchDomainFaceVer (Tess, DFaceVer);

  (*pDNeigh).nSize = ut_alloc_1d (7);

  for (i = 1; i <= 6; i++)
    {
      /* tmp[j] is the distance bw ver j and its poly centres. */
      tmp = ut_alloc_1d (DFaceVer[i][0] + 1);

      for (j = 1; j <= DFaceVer[i][0]; j++)
	{
	  ver = DFaceVer[i][j];

	  for (k = 0; k <= 3; k++)
	    {
	      poly = Tess.VerGerm[ver][k];

	      if (poly >= 1)
		{
		  tmp[j] =
		    threeDDoubleDist (Tess.VerCoo[ver], Tess.CenterCoo[poly],
				      0);
		  break;
		}
	    }
	}

      (*pDNeigh).nSize[i] =
	s[i] + pow (-1, i) * oneDDoubleMax (tmp, 1, DFaceVer[i][0]);

      oneDDoubleFree (tmp);
    }

  /* Calculating the volume */
  CalcNeighVol (pDNeigh);

  return;
}


/* DNeighGermsPer defines:
 * nGermsCoo, nGermQty, GermToGerm
 */
extern void
DNeighGermsPer (struct DNEIGH *pDNeigh)
{
  int i, j, id;
  int * per = ut_alloc_1d_int (4);
  double * coo = ut_alloc_1d (4);
  /* shortcuts: */
  double *s = (*pDNeigh).Size;	/* domain size          */
  double *ns = (*pDNeigh).nSize;	/* domain neigh size    */

  /* Distributing the germs that are into the sd+neigh zone
   * these are the max sizes.
   */
  (*pDNeigh).nGermsCoo = ut_alloc_2d (26 * (*pDNeigh).N + 1, 4);
  (*pDNeigh).GermToGerm = ut_alloc_1d_int (26 * (*pDNeigh).N + 1);

  id = 0;
  for (per[3] = -1; per[3] <= 1; per[3]++)	/* Z-periodicity */
    for (per[2] = -1; per[2] <= 1; per[2]++)	/* Y-periodicity */
      for (per[1] = -1; per[1] <= 1; per[1]++)	/* X-periodicity */
	if (per[3] == 0 && per[2] == 0 && per[1] == 0)
	  continue;
	else
	  for (i = 1; i <= (*pDNeigh).N; i++)
	    {
	      for (j = 1; j <= 3; j++)
		coo[j] = (*pDNeigh).GermsCoo[i][j] +
		  per[j] * (s[2 * j] - s[2 * j - 1]);

	      if ((coo[1] > ns[1] && coo[1] < s[1])
		  || (coo[1] < ns[2] && coo[1] > s[2])
		  || (coo[2] > ns[3] && coo[2] < s[3])
		  || (coo[2] < ns[4] && coo[2] > s[4])
		  || (coo[3] > ns[5] && coo[3] < s[5])
		  || (coo[3] < ns[6] && coo[3] > s[6]))
		{
		  id++;
		  for (j = 1; j <= 3; j++)
		    (*pDNeigh).nGermsCoo[id][j] = coo[j];

		  (*pDNeigh).GermToGerm[id] = i;
		}
	    }

  (*pDNeigh).nN = id;

  oneDIntFree (per);
  oneDDoubleFree (coo);

  return;
}

extern void
DNeighGermsSD (struct DNEIGH *pDNeigh)
{
  int i, j, id;
  double trash;
  double * coo = ut_alloc_1d (4);
  int RandId;

  double *s = (*pDNeigh).Size;
  double *ns = (*pDNeigh).nSize;
  gsl_rng *r;


  /* Calculation of the number of germs into the neighbourhood zone
   * the mean is NeighRealVol*(*pGermSet).NDensity.
   * the rand seed is the same than for the inside germ, and
   * we take the second value (the first was for the inside germs).
   */
  (*pDNeigh).nN = PoissonNb ((*pDNeigh).nVol * (double) (*pDNeigh).NDensity,
			     (*pDNeigh).Random, 2);

  (*pDNeigh).nGermsCoo = ut_alloc_2d ((*pDNeigh).nN + 1, 4);
  (*pDNeigh).nGermRandId = ut_alloc_1d_int ((*pDNeigh).nN + 1);

  /* Calculating the neighbourhood germs coo */

  r = gsl_rng_alloc (gsl_rng_ranlxd2);
  gsl_rng_set (r, (*pDNeigh).Random);

  /* skipping the 10*GermSet.NDensity first values:
   * near GermSet.NDensity are used for the inside germs
   * we skip 2*GermSet.NDensity more to avoid correllation
   * bw the germs positions of:
   * -MM -n 10 -id 1 -ttype 0 (case 1)
   * -MM -n 10 -id 1 -ttype 2 (case 2)
   * indeed, assuming that the second case has 7 inside germs, 
   * and if we do not skip 10*10 but 10, the position of the 8th
   * germ of case 1 would be correlated to the 1st neighbourhood
   * germ of case 2 
   * ((one more condition is that the 1st germ is
   * in the neigh zone (ie outside the domain))).
   * ... let's keep everything perfect :-) ...
   */

  for (i = 1; i <= 10 * (*pDNeigh).NDensity; i++)
    trash = gsl_rng_uniform (r);

  id = 0;
  RandId = 10 * (*pDNeigh).NDensity;
  while (id != (*pDNeigh).nN)
    {
      /* calculating tmp coo */
      for (j = 1; j <= 3; j++)
	{
	  coo[j] =
	    ns[2 * j - 1] + gsl_rng_uniform (r) * (ns[2 * j] - ns[2 * j - 1]);
	  RandId++;
	}

      /* if the point is not within the domain */
      if ((coo[1] < s[1] || coo[1] > s[2])	/* actually: */
	  || (coo[2] < s[3] || coo[2] > s[4])	/* s[2i-1]=0 */
	  || (coo[3] < s[5] || coo[3] > s[6]))	/* s[2i]  =1 */
	{
	  id++;			/* registering! */
	  for (j = 1; j <= 3; j++)
	    (*pDNeigh).nGermsCoo[id][j] = coo[j];

	  (*pDNeigh).nGermRandId[id] = RandId - 2;
	}
    }

  gsl_rng_free (r);
  oneDDoubleFree (coo);

  return;
}


/* DNeigh4EmpD defines the neighbouhood for an empty domain (no germ inside
 * it):
 * nSize, nN, nGermsCoo.
 */
void
DNeigh4EmpD (struct DNEIGH *pDNeigh)
{
  int i, j, id;
  int tmpnN = 0, tmpnN2 = 0, RandId;
  double tmpnVol, tmpnVol2;
  double **tmpGermsCoo;
  int *tmpRandId;
  double trash, dist;
  double * VerToGermDist = ut_alloc_1d (9);
  double * coo = ut_alloc_1d (4);
  double * tmpnSize = ut_alloc_1d (7);
  double ** VerCoo = ut_alloc_2d (9, 3);

  double *s = (*pDNeigh).Size;	/* MUST be [0,1]**3 */
  gsl_rng *r;

  VerCoo[1][0] = 0;
  VerCoo[1][1] = 0;
  VerCoo[1][2] = 0;
  VerCoo[2][0] = 1;
  VerCoo[2][1] = 0;
  VerCoo[2][2] = 0;
  VerCoo[3][0] = 1;
  VerCoo[3][1] = 1;
  VerCoo[3][2] = 0;
  VerCoo[4][0] = 0;
  VerCoo[4][1] = 1;
  VerCoo[4][2] = 0;
  VerCoo[5][0] = 0;
  VerCoo[5][1] = 0;
  VerCoo[5][2] = 1;
  VerCoo[6][0] = 1;
  VerCoo[6][1] = 0;
  VerCoo[6][2] = 1;
  VerCoo[7][0] = 1;
  VerCoo[7][1] = 1;
  VerCoo[7][2] = 1;
  VerCoo[8][0] = 0;
  VerCoo[8][1] = 1;
  VerCoo[8][2] = 1;
  tmpnSize[1] = tmpnSize[3] = tmpnSize[5] = 0;
  tmpnSize[2] = tmpnSize[4] = tmpnSize[6] = 1;

  for (i = 1; i <= 8; i++)
    VerToGermDist[i] = 1e32;

  /* Calculating the neighbourhood germs coo */

  r = gsl_rng_alloc (gsl_rng_ranlxd2);
  gsl_rng_set (r, (*pDNeigh).Random);

  for (i = 1; i <= 10 * (*pDNeigh).NDensity; i++)
    trash = gsl_rng_uniform (r);
  RandId = 10 * (*pDNeigh).NDensity;

  /* The neigh size is increased until there is at least one germ
   * inside it. */
  while (tmpnN == 0)
    {
      for (i = 1; i <= 6; i++)
	tmpnSize[i] += pow (-1, i);

      tmpnVol = (tmpnSize[2] - tmpnSize[1]) *
	(tmpnSize[4] - tmpnSize[3]) * (tmpnSize[6] - tmpnSize[5]) - 1;

      tmpnN = PoissonNb (tmpnVol * (double) (*pDNeigh).NDensity,
			 (*pDNeigh).Random, 2);
    }

  tmpGermsCoo = ut_alloc_2d (tmpnN + 1, 4);
  tmpRandId = ut_alloc_1d_int (tmpnN + 1);

  /* distributing germs */
  id = 0;
  while (id != tmpnN)
    {
      /* calculating tmp coo */
      for (j = 1; j <= 3; j++)
	{
	  coo[j] = tmpnSize[2 * j - 1] +
	    gsl_rng_uniform (r) * (tmpnSize[2 * j] - tmpnSize[2 * j - 1]);
	  RandId++;
	}

      /* if the point is not within the domain */
      if ((coo[1] < s[1] || coo[1] > s[2])
	  || (coo[2] < s[3] || coo[2] > s[4])
	  || (coo[3] < s[5] || coo[3] > s[6]))
	{
	  id++;			/* registering! */
	  for (j = 1; j <= 3; j++)
	    tmpGermsCoo[id][j] = coo[j];

	  tmpRandId[id] = RandId - 2;
	}
    }

  /* Calculating the minimum distances bw the vertices and the germs. 
   * to determine the real neighbouhood size 
   */
  for (i = 1; i <= 8; i++)
    for (j = 1; j <= tmpnN; j++)
      {
	dist = threeDDoubleDist (VerCoo[i], tmpGermsCoo[j] + 1, 0);
	VerToGermDist[i] = DoubleMin (VerToGermDist[i], dist);
      }

  (*pDNeigh).nSize = ut_alloc_1d (7);

  /* Calculating the real neighbouhood size from tmpnSize, the size of the
   * domain, and the germs distribution.
   */
  for (i = 1; i <= 3; i++)
    {
      /* lower bounds */
      (*pDNeigh).nSize[2 * i - 1] = tmpnSize[2 * i - 1];

      for (j = 1; j <= 8; j++)
	(*pDNeigh).nSize[2 * i - 1] =
	  DoubleMin ((*pDNeigh).nSize[2 * i - 1],
		     VerCoo[j][i] - VerToGermDist[j]);

      /* upper bounds */
      (*pDNeigh).nSize[2 * i] = tmpnSize[2 * i];

      for (j = 1; j <= 8; j++)
	(*pDNeigh).nSize[2 * i] =
	  DoubleMax ((*pDNeigh).nSize[2 * i],
		     VerCoo[j][i] + VerToGermDist[j]);
    }
  CalcNeighVol (pDNeigh);

  /* parameters for the extra neighbouring zone: vol and N */
  tmpnVol2 = ((*pDNeigh).nSize[2] - (*pDNeigh).nSize[1]) *
    ((*pDNeigh).nSize[4] - (*pDNeigh).nSize[3]) *
    ((*pDNeigh).nSize[6] - (*pDNeigh).nSize[5]) - tmpnVol - 1;

  tmpnN2 =
    PoissonNb (tmpnVol2 * (double) (*pDNeigh).NDensity, (*pDNeigh).Random, 3);

  /* Recording the real nN, nGermsCoo and nGermRandId */
  (*pDNeigh).nN = tmpnN + tmpnN2;
  (*pDNeigh).nGermsCoo = ut_alloc_2d ((*pDNeigh).nN + 1, 4);
  (*pDNeigh).nGermRandId = ut_alloc_1d_int ((*pDNeigh).nN + 1);

  for (i = 1; i <= tmpnN; i++)
    {
      for (j = 1; j <= 3; j++)
	(*pDNeigh).nGermsCoo[i][j] = tmpGermsCoo[i][j];

      (*pDNeigh).nGermRandId[i] = tmpRandId[i];
    }

  id = tmpnN;
  while (id != (*pDNeigh).nN)
    {
      /* calculating tmp coo */
      for (j = 1; j <= 3; j++)
	{
	  coo[j] = (*pDNeigh).nSize[2 * j - 1] +
	    gsl_rng_uniform (r) * ((*pDNeigh).nSize[2 * j] -
				   (*pDNeigh).nSize[2 * j - 1]);
	  RandId++;
	}

      /* if the point is not within the domain */
      if ((coo[1] < tmpnSize[1] || coo[1] > tmpnSize[2])
	  || (coo[2] < tmpnSize[3] || coo[2] > tmpnSize[4])
	  || (coo[3] < tmpnSize[5] || coo[3] > tmpnSize[6]))
	{
	  id++;			/* registering! */
	  for (j = 1; j <= 3; j++)
	    (*pDNeigh).nGermsCoo[id][j] = coo[j];

	  (*pDNeigh).nGermRandId[id] = RandId - 2;
	}
    }

  gsl_rng_free (r);

  ut_free_1d_int (tmpRandId);
  ut_free_2d (tmpGermsCoo, tmpnN + 1);

  ut_free_1d (VerToGermDist);
  ut_free_1d (coo);
  ut_free_1d (tmpnSize);
  ut_free_2d (VerCoo, 9);

  return;
}

==> neper-1.8.1.tgz: neper-1.8.1/src/neper_t/Miscellaneous/Miscellaneous2.c <==
/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"Miscellaneous.h"

void
DNeighSize (struct DNEIGH *pDNeigh)
{
  int i, j, k;
  int ver, poly;
  double *tmp;
  int **DFaceVer;
  double *s = (*pDNeigh).Size;

  struct GERMSET GermSet;
  struct TESS Tess;

  if ((*pDNeigh).N <= 0)
  {
    MessageWNC (2, 72, "The domain has no germ.");
    abort ();
  }

  if (((*pDNeigh).Size[1] >= (*pDNeigh).Size[2])
      || ((*pDNeigh).Size[3] >= (*pDNeigh).Size[4])
      || ((*pDNeigh).Size[5] >= (*pDNeigh).Size[6]))
  {
    MessageWNC (2, 72, "Bad domain size.");
    abort ();
  }

  /* Building GermSet */
  GermSet.N = (*pDNeigh).N;
  /* GermSet.Id=(*pDNeigh).Id; */
  GermSet.GermsCoo = ut_alloc_2d (GermSet.N + 1, 4);
  for (i = 1; i <= GermSet.N; i++)
  {
    for (j = 1; j <= 3; j++)
      GermSet.GermsCoo[i][j] = (*pDNeigh).GermsCoo[i][j];
  }

  /* Building tessellation to calculate the neighbourhood zone. */
  CreateTess (GermSet, (*pDNeigh).Size, &Tess);

  DFaceVer = ut_alloc_2d_int (7, Tess.VerQty + 1);
  SearchDomainFaceVer (Tess, DFaceVer);

  (*pDNeigh).nSize = ut_alloc_1d (7);

  for (i = 1; i <= 6; i++)
  {
    /* tmp[j] is the distance bw ver j and its poly centres. */
    tmp = ut_alloc_1d (DFaceVer[i][0] + 1);

    for (j = 1; j <= DFaceVer[i][0]; j++)
    {
      ver = DFaceVer[i][j];

      for (k = 0; k <= 3; k++)
      {
	poly = Tess.VerGerm[ver][k];

	if (poly >= 1)
	{
	  tmp[j] =
	    threeDDoubleDist (Tess.VerCoo[ver], Tess.CenterCoo[poly], 0);
	  break;
	}
      }
    }

    (*pDNeigh).nSize[i] =
      s[i] + pow (-1, i) * oneDDoubleMax (tmp, 1, DFaceVer[i][0]);

    oneDDoubleFree (tmp);
  }

  /* Calculating the volume */
  CalcNeighVol (pDNeigh);

  return;
}


/* DNeighGermsPer defines:
 * nGermsCoo, nGermQty, GermToGerm
 */
extern void
DNeighGermsPer (struct DNEIGH *pDNeigh)
{
  int i, j, id;
  int *per = ut_alloc_1d_int (4);
  double *coo = ut_alloc_1d (4);
  /* shortcuts: */
  double *s = (*pDNeigh).Size;	/* domain size          */
  double *ns = (*pDNeigh).nSize;	/* domain neigh size    */

  /* Distributing the germs that are into the sd+neigh zone
   * these are the max sizes.
   */
  (*pDNeigh).nGermsCoo = ut_alloc_2d (26 * (*pDNeigh).N + 1, 4);
  (*pDNeigh).GermToGerm = ut_alloc_1d_int (26 * (*pDNeigh).N + 1);

  id = 0;
  for (per[3] = -1; per[3] <= 1; per[3]++)	/* Z-periodicity */
    for (per[2] = -1; per[2] <= 1; per[2]++)	/* Y-periodicity */
      for (per[1] = -1; per[1] <= 1; per[1]++)	/* X-periodicity */
	if (per[3] == 0 && per[2] == 0 && per[1] == 0)
	  continue;
	else
	  for (i = 1; i <= (*pDNeigh).N; i++)
	  {
	    for (j = 1; j <= 3; j++)
	      coo[j] = (*pDNeigh).GermsCoo[i][j] +
		per[j] * (s[2 * j] - s[2 * j - 1]);

	    if ((coo[1] > ns[1] && coo[1] < s[1])
		|| (coo[1] < ns[2] && coo[1] > s[2])
		|| (coo[2] > ns[3] && coo[2] < s[3])
		|| (coo[2] < ns[4] && coo[2] > s[4])
		|| (coo[3] > ns[5] && coo[3] < s[5])
		|| (coo[3] < ns[6] && coo[3] > s[6]))
	    {
	      id++;
	      for (j = 1; j <= 3; j++)
		(*pDNeigh).nGermsCoo[id][j] = coo[j];

	      (*pDNeigh).GermToGerm[id] = i;
	    }
	  }

  (*pDNeigh).nN = id;

  oneDIntFree (per);
  oneDDoubleFree (coo);

  return;
}

extern void
DNeighGermsSD (struct DNEIGH *pDNeigh)
{
  int i, j, id;
  double trash;
  double *coo = ut_alloc_1d (4);
  int RandId;

  double *s = (*pDNeigh).Size;
  double *ns = (*pDNeigh).nSize;
  gsl_rng *r;


  /* Calculation of the number of germs into the neighbourhood zone
   * the mean is NeighRealVol*(*pGermSet).NDensity.
   * the rand seed is the same than for the inside germ, and
   * we take the second value (the first was for the inside germs).
   */
  (*pDNeigh).nN = PoissonNb ((*pDNeigh).nVol * (double) (*pDNeigh).NDensity,
			     (*pDNeigh).Random, 2);

  (*pDNeigh).nGermsCoo = ut_alloc_2d ((*pDNeigh).nN + 1, 4);
  (*pDNeigh).nGermRandId = ut_alloc_1d_int ((*pDNeigh).nN + 1);

  /* Calculating the neighbourhood germs coo */

  r = gsl_rng_alloc (gsl_rng_ranlxd2);
  gsl_rng_set (r, (*pDNeigh).Random);

  /* skipping the 10*GermSet.NDensity first values:
   * near GermSet.NDensity are used for the inside germs
   * we skip 2*GermSet.NDensity more to avoid correllation
   * bw the germs positions of:
   * -MM -n 10 -id 1 -ttype 0 (case 1)
   * -MM -n 10 -id 1 -ttype 2 (case 2)
   * indeed, assuming that the second case has 7 inside germs, 
   * and if we do not skip 10*10 but 10, the position of the 8th
   * germ of case 1 would be correlated to the 1st neighbourhood
   * germ of case 2 
   * ((one more condition is that the 1st germ is
   * in the neigh zone (ie outside the domain))).
   * ... let's keep everything perfect :-) ...
   */

  for (i = 1; i <= 10 * (*pDNeigh).NDensity; i++)
    trash = gsl_rng_uniform (r);

  id = 0;
  RandId = 10 * (*pDNeigh).NDensity;
  while (id != (*pDNeigh).nN)
  {
    /* calculating tmp coo */
    for (j = 1; j <= 3; j++)
    {
      coo[j] =
	ns[2 * j - 1] + gsl_rng_uniform (r) * (ns[2 * j] - ns[2 * j - 1]);
      RandId++;
    }

    /* if the point is not within the domain */
    if ((coo[1] < s[1] || coo[1] > s[2])	/* actually: */
	|| (coo[2] < s[3] || coo[2] > s[4])	/* s[2i-1]=0 */
	|| (coo[3] < s[5] || coo[3] > s[6]))	/* s[2i]  =1 */
    {
      id++;			/* registering! */
      for (j = 1; j <= 3; j++)
	(*pDNeigh).nGermsCoo[id][j] = coo[j];

      (*pDNeigh).nGermRandId[id] = RandId - 2;
    }
  }

  gsl_rng_free (r);
  oneDDoubleFree (coo);

  return;
}


/* DNeigh4EmpD defines the neighbouhood for an empty domain (no germ inside
 * it):
 * nSize, nN, nGermsCoo.
 */
void
DNeigh4EmpD (struct DNEIGH *pDNeigh)
{
  int i, j, id;
  int tmpnN = 0, tmpnN2 = 0, RandId;
  double tmpnVol, tmpnVol2;
  double **tmpGermsCoo;
  int *tmpRandId;
  double trash, dist;
  double *VerToGermDist = ut_alloc_1d (9);
  double *coo = ut_alloc_1d (4);
  double *tmpnSize = ut_alloc_1d (7);
  double **VerCoo = ut_alloc_2d (9, 3);

  double *s = (*pDNeigh).Size;	/* MUST be [0,1]**3 */
  gsl_rng *r;

  VerCoo[1][0] = 0;
  VerCoo[1][1] = 0;
  VerCoo[1][2] = 0;
  VerCoo[2][0] = 1;
  VerCoo[2][1] = 0;
  VerCoo[2][2] = 0;
  VerCoo[3][0] = 1;
  VerCoo[3][1] = 1;
  VerCoo[3][2] = 0;
  VerCoo[4][0] = 0;
  VerCoo[4][1] = 1;
  VerCoo[4][2] = 0;
  VerCoo[5][0] = 0;
  VerCoo[5][1] = 0;
  VerCoo[5][2] = 1;
  VerCoo[6][0] = 1;
  VerCoo[6][1] = 0;
  VerCoo[6][2] = 1;
  VerCoo[7][0] = 1;
  VerCoo[7][1] = 1;
  VerCoo[7][2] = 1;
  VerCoo[8][0] = 0;
  VerCoo[8][1] = 1;
  VerCoo[8][2] = 1;
  tmpnSize[1] = tmpnSize[3] = tmpnSize[5] = 0;
  tmpnSize[2] = tmpnSize[4] = tmpnSize[6] = 1;

  for (i = 1; i <= 8; i++)
    VerToGermDist[i] = 1e32;

  /* Calculating the neighbourhood germs coo */

  r = gsl_rng_alloc (gsl_rng_ranlxd2);
  gsl_rng_set (r, (*pDNeigh).Random);

  for (i = 1; i <= 10 * (*pDNeigh).NDensity; i++)
    trash = gsl_rng_uniform (r);
  RandId = 10 * (*pDNeigh).NDensity;

  /* The neigh size is increased until there is at least one germ
   * inside it. */
  while (tmpnN == 0)
  {
    for (i = 1; i <= 6; i++)
      tmpnSize[i] += pow (-1, i);

    tmpnVol = (tmpnSize[2] - tmpnSize[1]) *
      (tmpnSize[4] - tmpnSize[3]) * (tmpnSize[6] - tmpnSize[5]) - 1;

    tmpnN = PoissonNb (tmpnVol * (double) (*pDNeigh).NDensity,
		       (*pDNeigh).Random, 2);
  }

  tmpGermsCoo = ut_alloc_2d (tmpnN + 1, 4);
  tmpRandId = ut_alloc_1d_int (tmpnN + 1);

  /* distributing germs */
  id = 0;
  while (id != tmpnN)
  {
    /* calculating tmp coo */
    for (j = 1; j <= 3; j++)
    {
      coo[j] = tmpnSize[2 * j - 1] +
	gsl_rng_uniform (r) * (tmpnSize[2 * j] - tmpnSize[2 * j - 1]);
      RandId++;
    }

    /* if the point is not within the domain */
    if ((coo[1] < s[1] || coo[1] > s[2])
	|| (coo[2] < s[3] || coo[2] > s[4])
	|| (coo[3] < s[5] || coo[3] > s[6]))
    {
      id++;			/* registering! */
      for (j = 1; j <= 3; j++)
	tmpGermsCoo[id][j] = coo[j];

      tmpRandId[id] = RandId - 2;
    }
  }

  /* Calculating the minimum distances bw the vertices and the germs. 
   * to determine the real neighbouhood size 
   */
  for (i = 1; i <= 8; i++)
    for (j = 1; j <= tmpnN; j++)
    {
      dist = threeDDoubleDist (VerCoo[i], tmpGermsCoo[j] + 1, 0);
      VerToGermDist[i] = DoubleMin (VerToGermDist[i], dist);
    }

  (*pDNeigh).nSize = ut_alloc_1d (7);

  /* Calculating the real neighbouhood size from tmpnSize, the size of the
   * domain, and the germs distribution.
   */
  for (i = 1; i <= 3; i++)
  {
    /* lower bounds */
    (*pDNeigh).nSize[2 * i - 1] = tmpnSize[2 * i - 1];

    for (j = 1; j <= 8; j++)
      (*pDNeigh).nSize[2 * i - 1] =
	DoubleMin ((*pDNeigh).nSize[2 * i - 1],
		   VerCoo[j][i] - VerToGermDist[j]);

    /* upper bounds */
    (*pDNeigh).nSize[2 * i] = tmpnSize[2 * i];

    for (j = 1; j <= 8; j++)
      (*pDNeigh).nSize[2 * i] =
	DoubleMax ((*pDNeigh).nSize[2 * i], VerCoo[j][i] + VerToGermDist[j]);
  }
  CalcNeighVol (pDNeigh);

  /* parameters for the extra neighbouring zone: vol and N */
  tmpnVol2 = ((*pDNeigh).nSize[2] - (*pDNeigh).nSize[1]) *
    ((*pDNeigh).nSize[4] - (*pDNeigh).nSize[3]) *
    ((*pDNeigh).nSize[6] - (*pDNeigh).nSize[5]) - tmpnVol - 1;

  tmpnN2 =
    PoissonNb (tmpnVol2 * (double) (*pDNeigh).NDensity, (*pDNeigh).Random, 3);

  /* Recording the real nN, nGermsCoo and nGermRandId */
  (*pDNeigh).nN = tmpnN + tmpnN2;
  (*pDNeigh).nGermsCoo = ut_alloc_2d ((*pDNeigh).nN + 1, 4);
  (*pDNeigh).nGermRandId = ut_alloc_1d_int ((*pDNeigh).nN + 1);

  for (i = 1; i <= tmpnN; i++)
  {
    for (j = 1; j <= 3; j++)
      (*pDNeigh).nGermsCoo[i][j] = tmpGermsCoo[i][j];

    (*pDNeigh).nGermRandId[i] = tmpRandId[i];
  }

  id = tmpnN;
  while (id != (*pDNeigh).nN)
  {
    /* calculating tmp coo */
    for (j = 1; j <= 3; j++)
    {
      coo[j] = (*pDNeigh).nSize[2 * j - 1] +
	gsl_rng_uniform (r) * ((*pDNeigh).nSize[2 * j] -
			       (*pDNeigh).nSize[2 * j - 1]);
      RandId++;
    }

    /* if the point is not within the domain */
    if ((coo[1] < tmpnSize[1] || coo[1] > tmpnSize[2])
	|| (coo[2] < tmpnSize[3] || coo[2] > tmpnSize[4])
	|| (coo[3] < tmpnSize[5] || coo[3] > tmpnSize[6]))
    {
      id++;			/* registering! */
      for (j = 1; j <= 3; j++)
	(*pDNeigh).nGermsCoo[id][j] = coo[j];

      (*pDNeigh).nGermRandId[id] = RandId - 2;
    }
  }

  gsl_rng_free (r);

  ut_free_1d_int (tmpRandId);
  ut_free_2d (tmpGermsCoo, tmpnN + 1);

  ut_free_1d (VerToGermDist);
  ut_free_1d (coo);
  ut_free_1d (tmpnSize);
  ut_free_2d (VerCoo, 9);

  return;
}
